#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Mise à jour du mot de passe LDAP
# Equipe Eole eole@ac-dijon.fr

import sys
from glob import glob
from os.path import join
from pyeole.process import system_code
from creole.eosfunc import load_container_var
from pyeole.service import service_code
from pyeole.ssha import ssha_encode
from pyeole.dict4ini import DictIni

container_vars = load_container_var()
# on demande le nouveau mot de passe
# si il n'est pas passé comme paramètre
try:
    new_pass = sys.argv[1]
    #reindex = True
except:
    new_pass = ""
    while len(new_pass) < 5:
        if new_pass != "":
            print "le mot de passe est trop court (5 caractères au moins)"
        new_pass = raw_input("\nNouveau mot de passe pour l'admin LDAP ? ")
    #reindex = False

# récupération du mot de passe chiffré
crypt_pwd = ssha_encode(new_pass)

# recupération de la liste des fichiers à modifier
modifs = {}
password_files = glob("/usr/share/eole/annuaire/password_files/*.ini")
for fic in password_files:
    for cle, opts in DictIni(fic):
        if cle not in modifs:
            modifs[cle] = opts
        else:
            # hack pour les fichier multi-conteneurs
            old_container = modifs[cle].get('container', '')
            new_container = opts.get('container', '')
            if type(old_container) is list:
                modifs[cle]['container'].append(new_container)
            else:
                modifs[cle]['container'] = [old_container, new_container]

all_containers = []
# parcours du dictionnaire des fichiers
for name, option in modifs.items():

    # lecture des options
    chaines_recherche = option['string']
    container_list = option.get('container', '')
    short_path = join(option['path'], name)
    if type(chaines_recherche) != list:
        chaines_recherche = (chaines_recherche,)
    if type(container_list) is not list:
        container_list = [container_list]

    for container in container_list:

        if container != '':
            all_containers.append(container)
            prefix = container_vars.get('container_path_%s' % container, '/')
            file_path = prefix+short_path
        else:
            file_path = short_path

        for chaine_recherche in chaines_recherche:

            # lecture du fichier de conf
            print "\nMise à jour du fichier %s..." % file_path
            conf_file = open(file_path)
            buffer_in = conf_file.readlines()
            conf_file.close()

            buffer_out = ""
            # on parcourt le fichier
            for line in buffer_in:
                # si la ligne actuelle contient la chaine recherchée
                if line.startswith(chaine_recherche):
                    print "définition du mot de passe trouvée : %s" % line.strip()
                    # on ajoute les possibles tabulations ou espaces en début de ligne
                    buffer_out += line[0:line.index(chaine_recherche)]
                    buffer_out += chaine_recherche
                    if name == 'slapd.conf':
                        buffer_out += crypt_pwd
                    else:
                        buffer_out += new_pass
                    if chaine_recherche.endswith('"'):
                        # si le mot de passe est entre guillemets, on ajoute le guillemet fermant
                        buffer_out += '"'
                    if name.endswith('.pl') or name.endswith('.pm') or name.endswith('.php'):
                        buffer_out += ';'
                    buffer_out += "\n"
                else:
                    # sinon, on recopie la ligne d'origine
                    buffer_out += line

            # sauvegarde du fichier de conf
            conf_file = open(file_path,"w")
            conf_file.write(buffer_out)
            conf_file.close()

        rights = option.get('rights')
        if rights:
            cmd = ['chown', rights, short_path]
            system_code(cmd, container=container)
        mode = option.get('mode')
        if mode:
            system_code(['chmod', mode, file_path])

# mise à jour du mot de passe LDAP pour samba
if 'fichier' in all_containers:
    print "\nMise à jour de Samba .."
    cmd = ["smbpasswd", '-w', new_pass]
    ret = system_code(cmd, container='fichier')
    if ret != 0:
        raise Exception, 'Erreur à la mise à jour Samba : %s' % str(ret[1:])

print "\nredémarrage des services, veuillez patienter ..."
# visiblement il vaut mieux le faire à chaque fois surtout pour ldap !
service_code('slapd', 'restart', 'annuaire')
#if reindex:
#    print "réindexation de l'annuaire..."
#    cmd = ['/usr/bin/sudo', '-u', 'openldap', '/usr/sbin/slapindex']
#    system_res(cmd)
if 'fichier' in all_containers:
    service_code('smbd', 'restart', 'fichier')
if 'mail' in all_containers:
    service_code('courier-authdaemon', 'restart', 'mail')

